<?php
/*
Plugin Name: LMB^Box WordPress Plugin API
Plugin URI: http://lmbbox.com/projects/wordpress-plugin-api/
Description: A WordPress Plugin API class that allows plugin developers to have a standard management class for their plugin.
Author: Thomas Montague
Version: 0.1
Author URI: http://lmbbox.com/
*/


/**
 * LMB^Box WordPress Plugin API class
 *
 * Use this class to be the base to plugins for WordPress.
 *
 * @category   PluginAPI
 * @package    LMB^Box WordPress Plugin API
 * @author     Thomas Montague <lmbbox@gmail.com>
 * @copyright  2008 LMB^Box
 * @license    http://opensource.org/licenses/gpl-license.php GNU Public License
 * @version    Release: 0.1
 * @link       http://lmbbox.com/projects/wordpress-plugin-api/
 */
class WordPress_Plugin_API {

	// public Variables
	var $options						= array();

	// private const Variables
	var $__CLASS						= 'LMB^Box WordPress Plugin API';
	var $__SITE							= 'http://lmbbox.com/projects/wordpress-plugin-api/';
	var $__VERSION						= '0.1';
	var $__BUILD						= '20080413';
	var $__PARENT_MENUS					= array(
												'topmenu'		=> 'admin.php',
												'dashboard'		=> 'index.php',
												'write'			=> 'post-new.php',
												'manage'		=> 'edit.php',
												'comments'		=> 'edit-comments.php',
												'blogroll'		=> 'link-manager.php',
												'presentation'	=> 'themes.php',
												'plugins'		=> 'plugins.php',
												'users'			=> 'users.php',
												'options'		=> 'options-general.php'
											);
	var $__ACCESS_LEVELS				= array(
												'administrator'		=> 8,
												'editor'			=> 3,
												'author'			=> 2,
												'contributor'		=> 1,
												'subscriber'		=> 0
											);

	// private Variables
	var $__plugin_name					= NULL;
	var $__plugin_version				= NULL;
	var $__plugin_folder				= NULL;
	var $__plugin_file					= NULL;
	var $__menu_pages					= array();
	var $__meta_boxes					= array();



	/**
	 * Sets inital settings for class
	 *
	 * PHP 5 constructor.
	 *
	 * @param string $name  the display name of plugin
	 * @param string $version  the version of the plugin
	 * @param string $folder  the folder name where the plugin file is located
	 * @param string $file  the file name of the plugin
	 *
	 * @return void
	 *
	 * @access public
	 */
	function __construct($name, $version, $folder, $file) {
		$this->__plugin_name = $name;
		$this->__plugin_version = $version;
		$this->__plugin_folder = $folder;
		$this->__plugin_file = $file;

		// Needs to be set in the class extending this API class
		$this->__setup_options();
		$this->__setup_menu_pages();

		// Handle Options ...
		$this->__get_options();
		$this->__save_options();

		// Add WordPress Actions and Filters
//		register_activation_hook($this->__plugin_folder . '/' . $this->__plugin_file, array(&$this, 'activate'));
//		register_deactivation_hook($this->__plugin_folder . '/' . $this->__plugin_file, array(&$this, 'deactivate'));

		add_action('admin_menu', array(&$this, '__add_menu_pages'));
	}

	/**
	 * Sets inital settings for class
	 *
	 * PHP 4 constructor. Just calls PHP 5 constructor.
	 *
	 * @param string $name  the display name of plugin
	 * @param string $version  the version of the plugin
	 * @param string $folder  the folder name where the plugin file is located
	 * @param string $file  the file name of the plugin
	 *
	 * @return void
	 *
	 * @access public
	 */
	function WordPress_Plugin_API($name, $version, $folder, $file) {
		$this->__construct($name, $version, $folder, $file);
	}

	/**
	 * Sets up plugin options
	 *
	 * Plugin extending this class should override this function. All calls
	 * to __add_option() function should be made here.
	 *
	 * @return void
	 *
	 * @access protected
	 */
	function __setup_options() {}

	/**
	 * Sets up menu pages
	 *
	 * Plugin extending this class should override this function. All calls
	 * to __add_menu_page() and __do_meta_boxes() functions should be made here.
	 *
	 * @return void
	 *
	 * @access protected
	 */
	function __setup_menu_pages() {}

	/**
	 * Uninstalls plugin from WordPress
	 *
	 * Called to delete stored options, deactivate plugin, and redirect to 
	 * WordPress plugin deactivated page.
	 *
	 * @return void
	 *
	 * @access private
	 */
	function __uninstall_plugin() {
		delete_option($this->__plugin_folder);
		deactivate_plugins($this->__plugin_folder . '/' . $this->__plugin_file);
		wp_redirect('plugins.php?deactivate=true');
	}

	/**
	 * Adds an option to the options array
	 *
	 * Creates a new key in the options array with the params.
	 *
	 * @param string $name  the name of option
	 * @param string $value  the value of the option
	 * @param string $description  the description of the option
	 * @param string $type  the input type of the option
	 * @param array $values  the possiable values of the option
	 * @param string $callback  the callback display function for the option
	 *
	 * @return void
	 *
	 * @access protected
	 */
	function __add_option($name, $value, $description, $type, $values = NULL, $callback = '__display_option') {
		$this->options[$name] = array(
										'value' 		=> $value,
										'values'		=> $values,
										'description' 	=> $description,
										'type' 			=> $type,
										'callback'		=> $callback
									);
	}

	/**
	 * Gets options stored in the database
	 *
	 * Merges current options array with options stored in database.
	 *
	 * @return bool
	 *
	 * @access protected
	 */
	function __get_options() {
		$options = get_option($this->__plugin_folder);
		if (is_array($options) && !empty($options)) {
			$this->options = (is_array($this->options) && !empty($this->options)) ? array_merge($this->options, $options) : $options;
			return TRUE;
		} else { return FALSE; }
	}

	/**
	 * Saves options to the database
	 *
	 * Saves current options array to in database. Should only be called
	 * when first adding option to database.
	 *
	 * @return bool
	 *
	 * @access protected
	 */
	function __save_options() {
		if (is_array($this->options) && !empty($this->options)) {
			add_option($this->__plugin_folder, $this->options, $this->__plugin_name . $this->__plugin_version);
			return TRUE;
		} else { return FALSE; }
	}

	/**
	 * Updates options to the database
	 *
	 * Updates current options array with options param and updates to
	 * in database.
	 *
	 * @param array $options  the options to update with values
	 *
	 * @return bool
	 *
	 * @access private
	 */
	function __update_options($options) {
		if (!is_array($this->options) && empty($this->options)) {
			if ($this->__get_options() === FALSE) { return FALSE; }
		}

		if (is_array($options) && !empty($options)) {
			foreach ($this->options as $option_name => $option_values) {
				if (isset($options[$option_name])) {
					$this->options[$option_name]['value'] = $options[$option_name];
				}
			}
			return update_option($this->__plugin_folder, $this->options);
		} else { return FALSE; }
	}

	/**
	 * Resets options to defaults
	 *
	 * Resets current options array with options param and updates to
	 * in database.
	 *
	 * @param array $options  the options to reset with values
	 *
	 * @return bool
	 *
	 * @access private
	 */
	function __reset_options($options) {
		if (!is_array($this->options) && empty($this->options)) {
			if ($this->__get_options() === FALSE) { return FALSE; }
		}

		if (is_array($options) && !empty($options)) {
			$current_options = $this->options;
			unset($this->options);
			$this->__setup_options();
			$default_options = $this->options;
			$this->options = $current_options;

			foreach ($this->options as $option_name => $option_values) {
				if (isset($options[$option_name])) {
					$this->options[$option_name]['value'] = $default_options[$option_name]['value'];
				}
			}
			return update_option($this->__plugin_folder, $this->options);
		} else { return FALSE; }
	}

	/**
	 * Deletes options from database
	 *
	 * Deletes stored options from database
	 *
	 * @return bool
	 *
	 * @access private
	 */
	function __delete_options() {
		return delete_option($this->__plugin_folder);
	}


	/**
	 * Adds a menu page to the __menu_pages array
	 *
	 * Creates a new key in the __menu_pages array with the params.
	 *
	 * @param string $page_name  the name of the menu page
	 * @param string $page_title  the page display title of the menu page
	 * @param string $menu_title  the menu display title of the menu page
	 * @param int|string $access_level  the access level of the menu page
	 * @param string $parent  the parent page of the menu page
	 * @param bool $show_uninstall  whether to display the uninstall button or not
	 * @param string $file  the file name of the menu page
	 *
	 * @return void
	 *
	 * @access protected
	 */
	function __add_menu_page($page_name, $page_title, $menu_title, $access_level, $parent = 'plugins', $show_uninstall = TRUE, $file = NULL) {
		if (is_null($file)) { $file = $this->__plugin_folder . '/' . $this->__plugin_file; }
		$this->__menu_pages[$page_name] = array('page_name' => $page_name, 'page_title' => $page_title, 'menu_title' => $menu_title, 'access_level' => $access_level, 'parent' => $this->__PARENT_MENUS[$parent], 'show_uninstall' => $show_uninstall, 'file' => $file);
	}

	/**
	 * Sets up menu pages from __menu_pages array
	 *
	 * Handles the 'admin_menu' action callback function.
	 *
	 * @return void
	 *
	 * @access private
	 */
	function __add_menu_pages() {
		if (is_array($this->__menu_pages) && !empty($this->__menu_pages)) {
			foreach ($this->__menu_pages as $menu) {
				if ($menu['parent'] == 'admin.php') {
					add_menu_page($menu['page_title'], $menu['menu_title'], $menu['access_level'], $menu['file'], array(&$this, '__display_menu_page'));
				} else {
					add_submenu_page($menu['parent'], $menu['page_title'], $menu['menu_title'], $menu['access_level'], $menu['file'], array(&$this, '__display_menu_page'));			
				}
			}
		}
	}


	/**
	 * Adds a meta box to the __meta_boxes array
	 *
	 * Creates a new key in the __meta_boxes array with the params.
	 *
	 * @param string $page  the page name of where the meta box is added
	 * @param string $context  the context name of where the meta box is added
	 * @param string $name  the name of the meta box
	 * @param string $title  the display title of the meta box
	 * @param string $option_names  a comma separated list of option's names
	 *
	 * @return void
	 *
	 * @access protected
	 */
	function __add_meta_box($page, $context, $name, $title, $option_names) {
		$this->__meta_boxes[$page][$context][$name] = array('title' => $title, 'options' => $option_names);
	}

	/**
	 * Displays meta boxes in __meta_boxes array
	 *
	 * Handles displaying meta boxes setup in __meta_boxes array.
	 *
	 * @param string $page  the page name of the meta box to display
	 * @param string $context  the context name of the meta box to display
	 *
	 * @return void
	 *
	 * @access private
	 */
	function __do_meta_boxes($page, $context) {
		if (is_array($this->__meta_boxes[$page][$context]) && !empty($this->__meta_boxes[$page][$context])) {
			foreach ($this->__meta_boxes[$page][$context] as $box ) {
				if (!empty($box['options'])) {
					if ($context != 'hidden') { echo '<tr valign="top"><th scope="row">' . $box['title'] . '</th><td>'; }
					$options = explode(',', $box['options']);
					foreach ($options as $name) {
						call_user_func(array(&$this, $this->options[$name]['callback']), $name);
					}
					if ($context != 'hidden') { echo '</td></tr>'; }
				}
			}
		}
	}




	/**
	 * Displays menu page in __menu_pages array
	 *
	 * Displays menu page in __menu_pages array by matching page_hook.
	 * Only one menu page per parent per file is allowed.
	 *
	 * @return void
	 *
	 * @access private
	 */
	function __display_menu_page() {
		// get the current page settings
		foreach ($this->__menu_pages as $menu_page) {
			if ($GLOBALS['page_hook'] == get_plugin_page_hook($menu_page['file'], $menu_page['parent'])) {
				break;
			}
		}

		if($_REQUEST['plugin_options_update']) {
			echo ($this->__update_options($_REQUEST) === TRUE) ? '<div id="message" class="updated fade"><p><strong>' .  __('Options saved.') . '</strong></p></div>' : '<div id="message" class="error"><p><strong>' .  __('Options failed to save.') . '</strong></p></div>';
		} else if($_REQUEST['plugin_options_reset']) {
			echo ($this->__reset_options($_REQUEST) === TRUE) ? '<div id="message" class="updated fade"><p><strong>' .  __('Options reset.') . '</strong></p></div>' : '<div id="message" class="error"><p><strong>' .  __('Options failed to reset.') . '</strong></p></div>';
		} else if($_REQUEST['plugin_options_uninstall'] && $menu_page['show_uninstall'] === TRUE) {
			$this->__uninstall_plugin();
		}

?>
	<div class="wrap" id="<?php echo $menu_page['page_name']; ?>-wrap">
		<h2><?php echo $menu_page['page_title']; ?></h2>
		<form method="post">
			<?php $this->__do_meta_boxes($menu_page['page_name'], 'hidden'); ?>
			<table class="form-table">
				<?php $this->__do_meta_boxes($menu_page['page_name'], 'normal'); ?>
			</table>

<?php //$this->__do_meta_boxes($menu_page['page_name'], 'sidebar'); ?>

			<p class="submit">
				<input type="submit" name="plugin_options_update" value="<?php _e('Save Changes') ?>" />
				<input type="submit" name="plugin_options_reset" value="<?php _e('Reset Options') ?>" onclick='return( confirm( "Do you really want to reset these options to their default values?" ) );' />
			<?php if ($menu_page['show_uninstall'] === TRUE) { ?>
				<br /><br />
				<input type="submit" name="plugin_options_uninstall" value="<?php _e('Uninstall Plugin') ?>" onclick='return( confirm( "Do you really want to uninstall this plugin and delete all stored information from the database?" ) );' />
			<?php } ?>
			</p>
		</form>
	</div>
<?php

   }


	/**
	 * Displays option form code
	 *
	 * Displays form code for option based on option type.
	 *
	 * @param string $option_name  the option name to display
	 *
	 * @return void
	 *
	 * @access private
	 */
	function __display_option($option_name) {
		$option = $this->options[$option_name];
		if (is_array($option) && !empty($option)) {
			switch ($option['type']) {
				case 'hidden':
					$option_code = '<input type="hidden" name="' . $option_name . '" id="' . $option_name . '" value="' . $option['value'] . '" />';
					break;
				case 'text':
					$option_code = '<input type="text" name="' . $option_name . '" id="' . $option_name . '" value="' . $option['value'] . '" /> ' . $option['description'] . '<br />';
					break;
				case 'password':
					$option_code = '<input type="password" name="' . $option_name . '" id="' . $option_name . '" value="' . $option['value'] . '" /> ' . $option['description'] . '<br />';
					break;
				case 'textarea':
					$option_code = '<p>' . $option['description'] . '</p><p><textarea name="' . $option_name . '" id="' . $option_name . '" style="width: 98%; font-size: 12px;" class="code" cols="60" rows="10"> ' . $option['value'] . '</textarea></p>';
					break;
				case 'checkbox':
					$option_code = '<label for="' . $option_name . '"><input type="checkbox" name="' . $option_name . '" id="' . $option_name . '"' . ($option['value'] ? ' checked="checked"' : '') . ' /> ' . $option['description'] . '</label><br />';
					break;
				case 'file':
					$option_code = '<input type="file" name="' . $option_name . '" id="' . $option_name . '" />' . $option['description'] . '<br />';
					break;
				case 'select':
					if (is_array($option['values']) && !empty($option['values'])) {
						$option_code = '<select name="' . $option_name . '" id="' . $option_name . '">';
						foreach ($option['values'] as $option_value => $option_value_label) {
							$option_code .= '<option value="' . $option_value . '"' . ($option['value'] == $option_value ? ' selected="selected"' : '') . '> ' . $option_value_label . '</option>';
						}
						$option_code .= '</select>';
						if (!empty($option['description'])) { $option_code .= '<p>' . $option['description'] . '</p>'; } else { echo '<br />'; }
					} else { $option_code = ''; }
					break;
				case 'radio':
					if (is_array($option['values']) && !empty($option['values'])) {
						foreach ($option['values'] as $option_value => $option_value_label) {
							$option_code .= '<p><label for="' . $option_name . '-' . $option_value . '"><input type="radio" name="' . $option_name . '" id="' . $option_name . '-' . $option_value . '" value="' . $option_value . '"' . ($option['value'] == $option_value ? ' checked="checked"' : '') . ' /> ' . $option_value_label . '</label></p>';
						}
						if (!empty($option['description'])) { $option_code .= '<p>' . $option['description'] . '</p>'; } else { echo '<br />'; }
					} else { $option_code = ''; }
					break;
				default:
					$option_code = '';
					break;
			}
			echo $option_code;
		}
	}


}

?>